// Copyright 2009, 2010 Christopher Smith
// 
// This file is part of smithers.twisty.
// 
// smithers.twisty is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
// 
// smithers.twisty is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
// 
// You should have received a copy of the GNU General Public License along with
// smithers.twisty. If not, see <http://www.gnu.org/licenses/>.

package smithers.twisty;

/**
 * Represents the <a
 * href="http://en.wikipedia.org/wiki/Rubik%27s_Cube#Move_notation">Singmaster
 * Notation</a> for the Rubik's Cube.
 * 
 * This class is currently limited to the 3x3x3 cube. It is also completely
 * untested.
 */
public class SingmasterNotation implements PuzzleNotation<RubiksCube> {
    
    /**
     * {@inheritDoc}
     */
    public Twist[] getTwists(String description, RubiksCube puzzle) {
        int s = puzzle.side();
        if (s != 3) return new Twist[0]; //TODO
        int i;
        char temp, tem2;
        int dir;
        int twists = 0;
        for (i=0; i<description.length(); i++) {
            temp = description.charAt(i);
            if (temp=='F' || temp=='B' || temp=='U' || temp=='D' || temp=='L' || temp=='R' ||
                temp=='f' || temp=='b' || temp=='u' || temp=='d' || temp=='l' || temp=='r' ||
                temp=='x' || temp=='y' || temp=='z')
                twists++;
        }
        
        Twist[] result = new Twist[twists];
        twists = 0;
        for (i=0; i<description.length(); i++) {
            temp = description.charAt(i);
            tem2 = i+1<description.length() ? description.charAt(i+1) : ' ';
            switch (tem2) {
                case '\'': dir=1; break;
                case '2': dir=2; break;
                default: dir=3; break;
            }
            switch (temp) {
                case 'U': result[twists++] = puzzle.getTwist(0, 0, 1, dir); break;
                case 'L': result[twists++] = puzzle.getTwist(1, 0, 1, dir); break;
                case 'F': result[twists++] = puzzle.getTwist(2, 0, 1, dir); break;
                case 'R': result[twists++] = puzzle.getTwist(3, 0, 1, dir); break;
                case 'B': result[twists++] = puzzle.getTwist(4, 0, 1, dir); break;
                case 'D': result[twists++] = puzzle.getTwist(5, 0, 1, dir); break;
                
                case 'u': result[twists++] = puzzle.getTwist(0, 0, 2, dir); break;
                case 'l': result[twists++] = puzzle.getTwist(1, 0, 2, dir); break;
                case 'f': result[twists++] = puzzle.getTwist(2, 0, 2, dir); break;
                case 'r': result[twists++] = puzzle.getTwist(3, 0, 2, dir); break;
                case 'b': result[twists++] = puzzle.getTwist(4, 0, 2, dir); break;
                case 'd': result[twists++] = puzzle.getTwist(5, 0, 2, dir); break;
                
                case 'x': result[twists++] = puzzle.getTwist(0, 0, s, dir); break;
                case 'y': result[twists++] = puzzle.getTwist(0, 0, s, dir); break;
                case 'z': result[twists++] = puzzle.getTwist(0, 0, s, dir); break;
            }
        }
        
        return result;
    }
    
}
