// Copyright 2009, 2010 Christopher Smith
// 
// This file is part of smithers.twisty.
// 
// smithers.twisty is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
// 
// smithers.twisty is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
// 
// You should have received a copy of the GNU General Public License along with
// smithers.twisty. If not, see <http://www.gnu.org/licenses/>.

package smithers.twisty;

/**
 * Extends the {@code TwistyPuzzle} interface to include new methods for twisty
 * puzzles for which a twist simply swaps stickers around. For instance, a
 * Rubik's Cube is such a puzzle, but a Square One is not.
 */
public interface StickerSwappingTwistyPuzzle extends TwistyPuzzle {
    
    /**
     * Gets the index of the sticker which originated in the current location
     * of the specified sticker.
     * 
     * @param index the sticker
     * @return the location
     * @see #getSticker(int)
     */
    public int getLocation(int index);
    
    /**
     * Gets the index of the sticker currently in the specified location. The
     * location is specified by the index of the sticker which originated
     * there.
     * 
     * @param loc the location
     * @return the sticker's index
     * @see #getLocation(int)
     */
    public int getSticker(int loc);
    
}
